rm(list=ls())
################################################################################
## adebat - classify.R
################################################################################
## version: 1.6.1
## created: the epoch, Francesca G. Henning <fontina82@hotmail.it>
## modified: 20100604, Damiano G. Preatoni <prea@uninsubria.it>
##           20100903, Damiano G. Preatoni <prea@uninsubria.it>
##                     added multiple file input
##                     added automatic install of required packages
##                     added sonobat 2.9 / 2.9.5 format detection logic
## Applies adebat classifiers to a measurement file made with SonoBat

################################################################################
## Please edit the three values below following instructions, then run the script
##
## samples.dir:
##   specify the path where SonoBat *-calldata.txt files are
##   e.g. samples.dir <- "C:/Program Files/SonoBat/Samples"
##   Remember that under Windows you must use the slash "/" instead of
##   the backslash "\"

samples.dir <- "V:/ARCHIVIO/audio/Chirotteri/ignoti/eolico_liguria2010/pantasina/p.ti_saggio/26aprile"

## classifier.dir:
##   specify the path where this script and the classifiers.RData file are located
##   e.g. classifier.dir <- "C:/Dati/classifiers"

classifier.dir <- "D:/adebat"

## habitat:
##   specify what kind of classifier you want to use, inserting one of the habitat
##   names below. Allowed values are:
##   "all_species" "alpine" "antropophilous" "forest" "open_areas" 
##   "phytophilous" "troglophilous" "urban" "annex_2" "rhinolophus" 
##   "myotis" "genus_1" "genus_2"       
##   For a detailed description of each classifier capabilities, please see README.classifiers

habitat <- "all_species"

## You have finished configuring the classification system.
## Now, you can run the whole script. Results will be saved as a .csv text file
## in the directory you have specified as samples.dir.
################################################################################
## do not modify anything below (unless you know what you're doing...)
################################################################################

#@TODO place here some sanity check to look for any missing package
necessary <- c('foreign', 'gtools', 'randomForest')
installed <- necessary %in% installed.packages()[, 'Package']
if (length(necessary[!installed]) >=1)
  install.packages(necessary[!installed])

require(foreign) # read.csv
require(gtools)  # smartbind
require(randomForest) #randomforest

workdir <- samples.dir
setwd(workdir)

# get a list of all .txt files in samples.dir and append them all
files.list <- list.files(path = workdir, pattern = ".*-CallData.txt" )
FIRST <- TRUE
for (f in files.list) {
  cat("processing ",f,"\n")
  in.f <- read.csv(paste(workdir,f,sep="/"), header=TRUE, dec=".", sep="\t", as.is=TRUE)
  if (FIRST == TRUE) {
    data <- in.f
    FIRST <- FALSE
  } else {
    data <- smartbind(data,in.f)
  }
  cat("\tadded ",nrow(in.f)," rows [total: ",nrow(data),"].\n")
}

# sonobat 2.9 or 2.9.5?
sonobat_version <- "2.9.5"
if (names(data)[1] == "filename") { # 2.9
  sonobat_version <- "2.9"
  field_names <- c("filename","parent directory","next directory up","time in file","preceding intrvl","calls/s",
                   "duration","Fc","Hi Freq","Low Freq","bandwidth","Freq@max","maxAmp%dur",
                   "Freq@knee","knee%dur","dominant slope","slope@Fc","end slope","steepest slope","lowest slope",
                   "total slope","HiF to knee slope","knee to Fc slope","Freq@ledge","ledge duration",
                   "Freq@ctr","Freq@+20dB","Freq@-20dB","Freq@+15dB","Freq@-15dB","Freq @+5dB","Freq@-5dB",
                   "20 dB bndwth","15 dB bndwth","5 dB bndwth","20 dB dur","15 dB dur","5 dB dur",
                   "ln exp A","exp B","ln(60-exp C)","1st Amp","2nd Amp","3rd Amp","4th Amp",
                   "quality","rel pwr of 2nd","rel pwr of 3rd","startF","endF","HiF minus startF","Fc minus endF")
  names.ok <- c( "filename","parent.directory","next.directory.up","time.in.file","preceding.intrvl","calls.s",
                 "durata","Fc","Fhi","Flo","bw","Fmax","maxAmp",
                 "Fflesso","knee.dur","dominant.slope","pendFc","end.slope","steepest.slope","lowest.slope",
                 "pend","HiF.to.knee.slope","knee.to.Fc.slope","Fspalla","ledge.duration",
                 "F0.5t","Freq..20dB","Freq..20dB.1","Freq..15dB","Freq..15dB.1","Freq...5dB","Freq..5dB",
                 "X20.dB.bndwth","X15.dB.bndwth","X5.dB.bndwth","X20.dB.dur","X15.dB.dur","X5.dB.dur",
                 "ln.exp.A","exp.B","ln.60.exp.C.","X1st.Amp","X2nd.Amp","X3rd.Amp","X4th.Amp",
                 "quality","rel.pwr.of.2nd","rel.pwr.of.3rd","Fstart","Fend","HiF.minus.startF","Fc.minus.endF")
}

if (names(data)[1] == "Filename") { # 2.9.5
  sonobat_version <- "2.9.5"
  field_names <- c("Filename","ParentDirectory","NextDirectoryUp","TimeInFile","PrecedingIntrvl","CallsPerSec",
                   "CallDuration","Fc","HiFreq","LowFreq","Bndwdth","FreqMaxPwr","PrcntMaxAmpDur","TimeFromMaxToFc",
                   "FreqKnee","PrcntKneeDur","StartF","EndF","DominantSlope","SlopeAtFc","StartSlope","EndSlope",
                   "SteepestSlope","LowestSlope","TotalSlope","HiFtoKnSlope","KneeToFcSlope","CummNmlzdSlp",
                   "HiFtoFcExpAmp","HiFtoFcDmp","KnToFcExpAmp","KnToFcDmp","HiFtoKnExpAmp","HiFtoKnDmp","FreqLedge",
                   "LedgeDuration","FreqCtr","FBak32dB","FFwd32dB","FBak20dB","FFwd20dB","FBak15dB","FFwd15dB",
                   "FBak5dB","FFwd5dB","Bndw32dB","Bndw20dB","Bndw15dB","Bndw5dB",
                   "DurOf32dB","DurOf20dB","DurOf15dB","DurOf5dB",
                   "Amp1stQrtl","Amp2ndQrtl","Amp3rdQrtl","Amp4thQrtl",
                   "Amp1stMean","Amp2ndMean","Amp3rdMean","Amp4thMean",
                   "LnExpA_StartAmp","LnExpB_StartAmp","AmpStartLn60ExpC",
                   "LnExpA_EndAmp","LnExpB_EndAmp","AmpEndLn60ExpC",
                   "AmpK@start","AmpK@end","AmpKurtosis","AmpSkew","AmpVariance","AmpMomentvAmpGausR2",
                   "Quality","HiFminusStartF","FcMinusEndF","RelPwr2ndTo1st","RelPwr3rdTo1st")
  names.ok <- c( "filename","parent.directory","next.directory.up","time.in.file","preceding.intrvl","calls.s",
                 "durata","Fc","Fhi","Flo","bw","Fmax","maxAmp","pctMaxAmp","TMaxFc",
                 "Fflesso","knee.dur","Fstart","Fend","dominant.slope","pendFc","start.slope","end.slope",
                 "steepest.slope","lowest.slope","pend","HiF.to.knee.slope","knee.to.Fc.slope","CummNmlzdSlp",
                 "HiFtoFcExpAmp","HiFtoFcDmp","KnToFcExpAmp","KnToFcDmp","HiFtoKnExpAmp","HiFtoKnDmp","Fspalla",
                 "ledge.duration","F0.5t","FBak32dB","FFwd32dB","Freq..20dB","Freq..20dB.1","Freq..15dB","Freq..15dB.1",
                 "Freq...5dB","Freq..5dB","X32.dB.bndwth","X20.dB.bndwth","X15.dB.bndwth","X5.dB.bndwth",
                 "X32.dB.dur","X20.dB.dur","X15.dB.dur","X5.dB.dur",
                 "Amp1stQrtl","Amp2ndQrtl","Amp3rdQrtl","Amp4thQrtl",
                 "Amp1stMean","Amp2ndMean","Amp3rdMean","Amp4thMean",                 
                 "ln.exp.A","exp.B","ln.60.exp.C.",
                 "LnExpA_EndAmp","LnExpB_EndAmp","AmpEndLn60ExpC",
                 "AmpK.start","AmpK.end","AmpKurtosis","AmpSkew","AmpVariance","AmpMomentvAmpGausR2",                    
                 "quality","HiF.minus.startF","Fc.minus.endF","rel.pwr.of.2nd","rel.pwr.of.3rd")
}

## reset field names
names(data) <- names.ok

# data conditioning (if any) here
data$species <- factor( substr(data$filename,1,4))
data$animal <- substr(data$filename,1,8)
data$id <- row.names(data)

# get a list of species acronyms
species.list <- levels(data$species)

# preset dataframe column names
#names(data) <- c( "filename","parent.directory","next.directory.up","time.in.file","preceding.intrvl","calls.s","durata","Fc","Fhi","Flo","bw","Fmax","maxAmp","Fflesso","knee.dur","dominant.slope","pendFc","end.slope","steepest.slope","lowest.slope","pend","HiF.to.knee.slope","knee.to.Fc.slope","Fspalla","ledge.duration","F0.5t","Freq..20dB","Freq..20dB.1","Freq..15dB","Freq..15dB.1","Freq...5dB","Freq..5dB","X20.dB.bndwth","X15.dB.bndwth","X5.dB.bndwth","X20.dB.dur","X15.dB.dur","X5.dB.dur","ln.exp.A","exp.B","ln.60.exp.C.","X1st.Amp","X2nd.Amp","X3rd.Amp","X4th.Amp","quality","rel.pwr.of.2nd","rel.pwr.of.3rd","Fstart","Fend","HiF.minus.startF","Fc.minus.endF","species","animal","id")


# load classifiers
load(file=paste(classifier.dir,"classifiers.RData",sep="/"))
rf.0 <- classifiers[[habitat]]

# apply classifier (probablity matrix)
res <- predict(rf.0,data,type="prob")
res <- data.frame(res)

# apply classifier again, maximum probability
result <- cbind(data,res*100)
res <- predict(rf.0,data)

# prepare final result & write it
result <- cbind(result,res)
# output file will be named as the directory containing it
outname <- strsplit(getwd(),"/")[[1]]
outname <- outname[length(outname)]
write.csv2(file=paste(outname,"classified.csv",sep="-"),result,row.names=FALSE)

## end of file ##
## SCRAP #######################################################################
# lookup table sample
# make up a lookup table
lookup.species <- function(sp) {
  species <- c("baba",          "epni",       "epse",       "hysa",   "misc",          "mybe",         "mybl","myca","myda","myem","mymy","myms","myna","nyle","nyno","piku","pina","pipi","plau","plas","plma","plsa","rheu","rhfe","rhhi","vemu")
  names <-   c("B.barbastellus","E.nilssonii","E.serotinus","H.savii","M.schreibersii","M.bechsteinii","M.blythii","M.capaccinii","M.daubentonii","M.emarginatus","M.myotis","M.mystacinus","M.nattereri","N.leisleri","N.noctula","P.kuhlii","P.pipistrellus","P.nathusii","P.auritus","P.austriacus","P.macrobullaris","P.sardus","R.euryale","R.ferrumequinum","R.hipposideros","V.murinus")
  lookup <- data.frame(species, names)
  result <- as.character(lookup[lookup$species==sp,2])
  return(result)
}

# esempio per car::recode
# "'baba'='B.barbastellus';'epni'='E.nilssonii';...
# quindi: 
# convertire il primo res (row 91) in dataframe
# names(res) <- recode(names(res),  "'baba'='B.barbastellus';'epni'='E.nilssonii';...



